// Global Variable Texture Viewer.
// Fisholith
// PS 2.0
// View and offset the textures that Construct allows you to refer to within effect code. Foreground, Background, Source, and Previous. (Handy for debugging effects, especially when used with the "Channel Select" effect.)

/* X */ // Verify PS version.
/* X */ // Remove unused items. (params, special vars, textures, etc.)
/* X */ // Verify Shader and Param descriptions.

//________________________________
////////////////////////////////// m: Special Construct Comment Modifiers
//^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

// Comment Modifiers: (enable with "#")
  //#BORDER-MODE
  // CROSS-SAMPLING

//________________________________
////////////////////////////////// m: Params
//^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

/* X */
//#PARAM float gfi_textureSelect 0 : Texture Select : [0] = Foreground.   [1] = Background.   [2] = Source.   [3] = Previous.
float gfi_textureSelect;
//#PARAM float gfi_shiftX 0 : Offset X : Offset in pixels.
float gfi_shiftX;
//#PARAM float gfi_shiftY 0 : Offset Y : Offset in pixels.
float gfi_shiftY;

//________________________________
////////////////////////////////// m: Special Construct Vars
//^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

/* X */
float boxWidth;
float boxHeight;

float boxLeft;
float boxRight;
float boxTop;
float boxBottom;

float pixelWidth;
float pixelHeight;

float hotspotX;
float hotspotY; 

float frameCounter;


//________________________________
////////////////////////////////// m: Texture Samplers
//^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

//====== Foreground

// Foreground texture
texture ForegroundTexture;

// Foreground sampler
sampler2D s2D_foreground = sampler_state {
    Texture = (ForegroundTexture);
    MinFilter = Point;
    MagFilter = Point;
    MipFilter = Point;
};

//====== Background 

// Background texture
texture BackgroundTexture;

// Background sampler
sampler2D s2D_background = sampler_state {
    Texture = (BackgroundTexture);
    MinFilter = Point;
    MagFilter = Point;
    MipFilter = Point;
};

//====== Source 

// Source texture
texture SourceTexture;

// Source sampler
sampler2D s2D_source = sampler_state {
    Texture = (SourceTexture);
    MinFilter = Point;
    MagFilter = Point;
    MipFilter = Point;
};

//====== Previous

// Previous texture
texture PreviousTexture;

// Previous sampler
sampler2D s2D_previous = sampler_state {
    Texture = (PreviousTexture);
    MinFilter = Point;
    MagFilter = Point;
    MipFilter = Point;
};


//________________________________
////////////////////////////////// m: Functions
//^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

//-----------------------------
// Convert pixel shader coords to Simple pixel coords.
//    Simple pixel coordinates:
//      Meaning 0,0 is the top left pixel, and 32,64
//      is the 32 pixesl right and 64 pixels down from 0,0.
float2 coordPsToPixel( float2 Tex )
{
	// Make pos relative to left edge, and scale relative to pixel width.
	Tex.x = ( ( Tex.x - boxLeft ) / pixelWidth );
	// Make pos relative to top edge, and scale relative to pixel height.
	Tex.y = ( ( Tex.y - boxTop ) / pixelHeight );	
	// Note: Values are floored to make sure they are exact pixel coords.
	return Tex;
}

//-----------------------------
// Convert Simple pixel coords to pixel shader coords.
//    Simple pixel coordinates:
//      Meaning 0,0 is the top left pixel, and 32,64
//      is the 32 pixesl right and 64 pixels down from 0,0.
float2 coordPixelToPs( float2 Tex )
{
	// Make pos relative to left edge, and scale relative to pixel width.
	Tex.x = ( ( Tex.x * pixelWidth ) + boxLeft );
	// Make pos relative to top edge, and scale relative to pixel height.
	Tex.y = ( ( Tex.y * pixelHeight ) + boxTop );
	// Note: Values are floored to make sure they are exact pixel coords.
	return Tex;
}

//-----------------------------
// Effect function
float4 EffectProcess( float2 Tex : TEXCOORD0 ) : COLOR0
{
	Tex = coordPsToPixel( Tex );
	Tex += float2( gfi_shiftX , gfi_shiftY );
	Tex = coordPixelToPs( Tex );
	
	// Get sample.
	float4 f4_texture = 0;
	if ( 0 == gfi_textureSelect )
	{
		f4_texture = tex2D( s2D_foreground , Tex.xy );
	}
	else if ( 1 == gfi_textureSelect )
	{
		f4_texture = tex2D( s2D_background , Tex.xy );
	}
	else if ( 2 == gfi_textureSelect )
	{
		f4_texture = tex2D( s2D_source , Tex.xy );
	}
	else // if ( 3 == gfi_textureSelect )
	{
		f4_texture = tex2D( s2D_previous , Tex.xy );
	}	
		
    return f4_texture;
}


//________________________________
////////////////////////////////// m: Techniques
//^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

//-----------------------------
// ConstructEffect
technique ConstructEffect
{
    pass p0
    {
        VertexShader = null;
        PixelShader = compile ps_2_0 EffectProcess();
    }
}

















