// Height Map Plus
// Jorge Fuente-Alba
// PS 2.0


// Foreground texture
texture ForegroundTexture;

// Background texture
texture BackgroundTexture;

//#PARAM float depth 10 : Depth : Depth of the heightmap.
float depth;

//#PARAM float specular 10 : specular shine : Determines size of light glare spots.
float specular;

//#PARAM float specint 1 : specular Intensity : Determines intensity of light glare spots.
float specint;


//#PARAM percent ambient 0.2 : Ambient light : Minimum light.
float ambient;

//#PARAM float lightx 0.5 : Light X : X co-ordinate of the light, zero is left.
float lightx;

//#PARAM float lighty 0.5 : Light Y : Y co-ordinate of the light, zero is up.
float lighty;

//#PARAM float lightz 0.5 : Light Z : Z co-ordinate of the light (height).
float lightz;

float pixelWidth;
float pixelHeight;

// Foreground sampler
sampler2D foreground = sampler_state {
    Texture = (ForegroundTexture);
    MinFilter = Point;
    MagFilter = Point;
    MipFilter = Point;
};

// OriginalTexture sampler
sampler2D background = sampler_state {
    Texture = (BackgroundTexture);
    MinFilter = Point;
    MagFilter = Point;
    MipFilter = Point;
};

// Effect function
float4 EffectProcess( float2 Tex : TEXCOORD0 ) : COLOR0
{
    // Find the average of the RGB and set all components to that value
    // creating a greyscale image
    float3 normalmap;
        float alpha = tex2D(foreground, Tex).a;
	float left = tex2D(foreground, Tex + float2(-1*pixelWidth,0)).r;
	float right = tex2D(foreground, Tex + float2(pixelWidth,0)).r;
	float up = tex2D(foreground, Tex + float2(0, -1*pixelHeight)).r;
	float down = tex2D(foreground, Tex + float2(0, pixelHeight)).r;
	normalmap.r = (left - right)*(depth/alpha);
	normalmap.g = (down-up)*(depth/alpha);
	normalmap.b = 0.5;
        normalmap = normalize(normalmap);

    // Screen formula
    float4 back = tex2D(background, Tex.xy);

    float3 light = normalize(float3(lightx - Tex.x, Tex.y - lighty, lightz));

    float diffuse = saturate(dot(normalmap,light));
    
    float4 lit = diffuse+ambient;
           lit = lit+(specint*(pow(diffuse,specular)));
           lit = lit*back*alpha;
           lit.a = alpha;

    return lit;
}

// ConstructEffect
technique ConstructEffect
{
    pass p0
    {
        VertexShader = null;
        PixelShader = compile ps_2_0 EffectProcess();
    }
}
